@@ -34,7 +34,11 @@
 #include <vector>
 #include <stdexcept>
 #include <csignal>
-#include <omp.h>
+#ifdef _OPENMP
+#   include <omp.h>
+#else
+#   define omp_get_thread_num() 0
+#endif
 
 using std::size_t;
 using std::uint64_t;
@@ -48,9 +52,6 @@ using std::int32_t;
 #endif
 
 #define restrict __restrict
-#ifndef _OPENMP
-#   define omp_get_thread_num() 0
-#endif
 
 #if defined(_FOR_R)
 extern "C" double ddot_(const int *n, const double *dx, const int *incx, const double *dy, const int *incy);
@@ -367,6 +368,9 @@ void calc_metrics
     uint64_t seed
 )
 {
+    #ifndef _OPENMP
+    nthreads = 1;
+    #endif
     nthreads = std::max(nthreads, 1);
     min_items_pool = std::max(min_items_pool, k_metrics);
     min_items_pool = std::max(min_items_pool, 2);
@@ -408,7 +412,12 @@ void calc_metrics
     #pragma omp parallel for schedule(dynamic) num_threads(nthreads) \
             private(pred_thread, ind_thread, test_bool_thread, A_thread, \
                     p_at_k_user, tp_at_k_user, r_at_k_user, ap_at_k_user, tap_at_k_user, \
-                    ndcg_at_k_user, hit_at_k_user, rr_at_k_user)
+                    ndcg_at_k_user, hit_at_k_user, rr_at_k_user) \
+            shared(p_at_k, tp_at_k, r_at_k, ap_at_k, tap_at_k, ndcg_at_k, hit_at_k, rr_at_k, roc_auc, pr_auc, \
+                   A, B, \
+                   Xtrain_csr_p, Xtrain_csr_i, Xtest_csr_p, Xtest_csr_i, Xtest_csr, \
+                   min_pos_test, min_items_pool, seed, \
+                   pred_buffer, ind_buffer, test_bool_buffer)
     for (int32_t user = 0; user < m; user++)
     {
         if (
